package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.api.feign.dto.ImUserInfo;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.auth.config.CustomerServiceConfig;
import com.yonge.cooleshow.biz.dal.dao.EmployeeDao;
import com.yonge.cooleshow.biz.dal.dao.StudentDao;
import com.yonge.cooleshow.biz.dal.dao.SubjectDao;
import com.yonge.cooleshow.biz.dal.dao.TeacherDao;
import com.yonge.cooleshow.biz.dal.dao.UserBindingTeacherDao;
import com.yonge.cooleshow.biz.dal.dto.search.QueryMyFollowSearch;
import com.yonge.cooleshow.biz.dal.dto.search.StudentSearch;
import com.yonge.cooleshow.biz.dal.entity.*;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.mapper.SysUserMapper;
import com.yonge.cooleshow.biz.dal.service.ImGroupService;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumMapper;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumPurchaseMapper;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumRefMapper;
import com.yonge.cooleshow.biz.dal.service.ImGroupMemberService;
import com.yonge.cooleshow.biz.dal.service.ImGroupService;
import com.yonge.cooleshow.biz.dal.service.ImUserFriendService;
import com.yonge.cooleshow.biz.dal.service.StudentService;
import com.yonge.cooleshow.biz.dal.service.StudentTotalService;
import com.yonge.cooleshow.biz.dal.service.im.ImGroupCoreService;
import com.yonge.cooleshow.biz.dal.service.SysConfigService;
import com.yonge.cooleshow.biz.dal.service.TeacherService;
import com.yonge.cooleshow.biz.dal.service.TenantInfoService;
import com.yonge.cooleshow.biz.dal.service.UserTenantAlbumRecordService;
import com.yonge.cooleshow.biz.dal.vo.MyFollow;
import com.yonge.cooleshow.biz.dal.vo.StudentHomeVo;
import com.yonge.cooleshow.biz.dal.vo.StudentVo;
import com.yonge.cooleshow.biz.dal.vo.TeacherVo;
import com.yonge.cooleshow.biz.dal.wrapper.StudentWrapper;
import com.yonge.cooleshow.biz.dal.wrapper.im.ImGroupWrapper;
import com.yonge.cooleshow.common.constant.SysConfigConstant;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.CacheNameEnum;
import com.yonge.cooleshow.common.enums.UserLockFlag;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.util.StringUtil;
import com.yonge.toolset.mybatis.support.PageUtil;
import com.yonge.toolset.utils.date.DateUtil;
import com.yonge.toolset.utils.easyexcel.ExcelDataReaderProperty;
import com.yonge.toolset.utils.string.ValueUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RMap;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.MessageFormat;
import java.text.ParseException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


@Slf4j
@Service
public class StudentServiceImpl extends ServiceImpl<StudentDao, Student> implements StudentService {
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private StudentTotalService totalService;
    @Autowired
    private ImUserFriendService imUserFriendService;
    @Resource
    private UserBindingTeacherDao userBindingTeacherDao;
    @Resource
    private TeacherDao teacherDao;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private SubjectDao subjectDao;
    @Autowired
    private TenantInfoService tenantInfoService;
    @Autowired
    private UserTenantAlbumRecordService userTenantAlbumRecordService;
    @Autowired
    private SysConfigService sysConfigService;

    @Autowired
    private ImGroupService imGroupService;

    @Autowired
    private ImGroupMemberService imGroupMemberService;

    @Autowired
    private TenantAlbumRefMapper tenantAlbumRefMapper;

    @Autowired
    private TenantAlbumMapper tenantAlbumMapper;

    @Autowired
    private EmployeeDao employeeDao;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private CustomerServiceConfig customerServiceConfig;

    @Override
    public StudentDao getDao() {
        return baseMapper;
    }

    @Autowired
    private ImGroupCoreService imGroupCoreService;

    @Override
    public StudentVo detail(Long userId) {
        return baseMapper.detail(userId);
    }

    @Override
    public StudentVo detailByPhone(String phone) {
        return baseMapper.detailByPhone(phone);
    }

    @Override
    public IPage<StudentVo> selectPage(IPage<StudentVo> page, StudentSearch studentSearch) {
        return page.setRecords(baseMapper.selectPage(page, studentSearch));
    }

    @Override
    public StudentHomeVo queryUserInfo(SysUser user) {
        StudentVo detail = detail(user.getId());
        StudentHomeVo studentHomeVo = new StudentHomeVo();
        BeanUtils.copyProperties(detail, studentHomeVo);

        try {
            ImGroupWrapper.ImUserInfo register = imGroupCoreService.register(user.getId().toString(),
                    ClientEnum.STUDENT.getCode(),
                    user.getUsername(),
                    user.getAvatar());
            studentHomeVo.setImToken(register.getImToken());
        } catch (Exception e) {
            throw new BizException("获取IMToken失败");
        }

        //身份证号、手机号脱敏
        studentHomeVo.setIdCardNo(ValueUtil.fuzzyIdCard(studentHomeVo.getIdCardNo()));
//        studentHomeVo.setPhone(ValueUtil.fuzzyMobile(studentHomeVo.getPhone()));
        String avatar = user.getAvatar();
        if (StringUtils.isNotBlank(detail.getAvatar())) {
            avatar = detail.getAvatar();
        }
        studentHomeVo.setHeardUrl(avatar);

        int num = DateUtil.daysBetween(new Date(), detail.getMembershipEndTime());
        studentHomeVo.setMembershipDays(num < 0 ? 0 : num);

        StudentTotal total = totalService.getTotalById(user.getId());
        if (null == total) {
            total = new StudentTotal();
        }

        studentHomeVo.setTenantAlbumStatus(0);
        studentHomeVo.setExerciseDays(null == total.getExerciseDays() ? 0 : total.getExerciseDays());
        studentHomeVo.setExerciseHours(null == total.getExerciseHours() ? 0 : total.getExerciseHours());
        studentHomeVo.setTestingNum(null == total.getTestingNum() ? 0 : total.getTestingNum());
        studentHomeVo.setFinshClassHours(null == total.getFinshHours() ? 0 : total.getFinshHours());
        studentHomeVo.setUnfinshClassHours(null == total.getUnfinshHours() ? 0 : total.getUnfinshHours());
        studentHomeVo.setStarTeacherNum(null == total.getStarTeacherNum() ? 0 : total.getStarTeacherNum());
        studentHomeVo.setMusicAlbumNum(null == total.getMusicAlbumNum() ? 0 : total.getMusicAlbumNum());
        studentHomeVo.setMusicSheetNum(null == total.getMusicSheetNum() ? 0 : total.getMusicSheetNum());

        // IM聊天用户ID
        studentHomeVo.setImUserId(imGroupService.getImUserId(String.valueOf(detail.getUserId()), ClientEnum.STUDENT.name()));

        // 判断是否是机构学生 机构学生 检测机构专辑购买记录
        TenantInfo tenantInfo = tenantInfoService.detail(detail.getTenantId());
        if (tenantInfo != null) {
            studentHomeVo.setTenantName(tenantInfo.getName());
            QueryWrapper<TenantAlbumRef> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(TenantAlbumRef::getTenantId, detail.getTenantId());
            List<TenantAlbumRef> albumRefs = tenantAlbumRefMapper.selectList(queryWrapper);
            if (!albumRefs.isEmpty()) {
                List<Long> albIds = albumRefs.stream().map(TenantAlbumRef::getTenantAlbumId).distinct().collect(Collectors.toList());
                QueryWrapper<TenantAlbum> query = new QueryWrapper<>();
                query.lambda().in(TenantAlbum::getId, albIds)
                        .eq(TenantAlbum::getStatus,true);
                Integer count = tenantAlbumMapper.selectCount(query);
                if (count > 0) {
                    studentHomeVo.setTenantAlbumStatus(1);
                }
            }
            UserTenantAlbumRecord record =
                    userTenantAlbumRecordService.getNewestByTenantIdAndUserId(tenantInfo.getId(), detail.getUserId(),
                            ClientEnum.STUDENT);
            if (record == null || record.getEndTime().getTime() < System.currentTimeMillis()) {
                studentHomeVo.setTenantAlbumFlag(YesOrNoEnum.NO);
            } else {
                studentHomeVo.setTenantAlbumStatus(2);
                studentHomeVo.setTenantAlbumFlag(YesOrNoEnum.YES);
                studentHomeVo.setTenantAlbumStartTime(record.getStartTime());
                studentHomeVo.setTenantAlbumEndTime(record.getEndTime());
            }

        }

        return studentHomeVo;
    }

    @Override
    public Integer setSubject(Long id, String subjectIds) {
        return baseMapper.setSubject(id, subjectIds);
    }

    @Override
    public List<Subject> querySubject(Long userId) {
        List<Subject> subjects = baseMapper.querySubject(userId);
        return subjects;
    }

    @Override
    public List<Subject> querySubjectItem(Long userId, String type) {
        //先查询自己的声部
        List<Subject> subjectList = querySubject(userId);

        List<Subject> cacheByType = getCacheByType(userId, type);
        if (CollectionUtils.isNotEmpty(cacheByType)) {
            subjectList.addAll(cacheByType);
        }
        return subjectList;
    }

    private List<Subject> getCacheByType(Long userId, String type) {
        RMap<String, List<Subject>> map = redissonClient.getMap(CacheNameEnum.STUDENT_SUBJECT_ITEM.getRedisKey(userId));
        //设置过期时间
        map.expire(CacheNameEnum.STUDENT_SUBJECT_ITEM.getDuration().toMinutes(),
                TimeUnit.MINUTES);

        if (StringUtil.isEmpty(type)) {
            List<Subject> subjectList = map.get("ALL");
            if (CollectionUtils.isEmpty(subjectList)) {
                subjectList = baseMapper.querySubjectItem(userId, null);
                map.put("ALL", subjectList);
            }
            return subjectList;
        } else {
            List<Subject> subjectList = map.get(type);
            if (CollectionUtils.isEmpty(subjectList)) {
                subjectList = baseMapper.querySubjectItem(userId, type);
                map.put(type, subjectList);
            }
            return subjectList;
        }
    }

    @Override
    public IPage<MyFollow> queryMyFollow(IPage<MyFollow> page, QueryMyFollowSearch query) {
        List<MyFollow> teacherVos = baseMapper.queryMyFollow(page, query);
        return page.setRecords(teacherVos);
    }

    @Override
    public HttpResponseResult<Map<String, TeacherVo>> bindTeacher(String phone, Long userId, Boolean isUpdate) {
        Map<String, TeacherVo> resMap = new HashMap<>();
        TeacherVo old = userBindingTeacherDao.getBindTeacherByPhone(phone);
        if (null != old) {
            //身份证号、手机号脱敏
            old.setIdCardNo(ValueUtil.fuzzyIdCard(old.getIdCardNo()));
            old.setPhone(ValueUtil.fuzzyMobile(old.getPhone()));
        }
        resMap.put("old", old);

        StudentVo studentVo = detailByPhone(phone);

        if (null == studentVo) {
            return HttpResponseResult.failed("未找到用户信息");
        }
        TeacherVo detail = teacherDao.detail(userId);
        if (null != detail) {
            //身份证号、手机号脱敏
            detail.setIdCardNo(ValueUtil.fuzzyIdCard(detail.getIdCardNo()));
            detail.setPhone(ValueUtil.fuzzyMobile(detail.getPhone()));
        }
        if (null == detail) {
            return HttpResponseResult.failed("未找老师信息");
        }
        if (UserLockFlag.LOCKED.equals(detail.getLockFlag())) {
            return HttpResponseResult.failed("老师被锁定");
        }
        if (null == old || (null != isUpdate && isUpdate)) {
            //更新
            //删除用户绑定
            userBindingTeacherDao.delete(Wrappers.<UserBindingTeacher>lambdaQuery()
                    .eq(UserBindingTeacher::getStudentId, studentVo.getUserId()));

            UserBindingTeacher userBindingTeacher = new UserBindingTeacher();
            userBindingTeacher.setTeacherId(userId);
            userBindingTeacher.setStudentId(studentVo.getUserId());
            userBindingTeacherDao.insert(userBindingTeacher);

            //学生老师增加好友关系
            Set<Long> studentIds = new HashSet<>();
            studentIds.add(studentVo.getUserId());
            imUserFriendService.saveUserFriend(userId, studentIds);
        }
        resMap.put("now", detail);
        return HttpResponseResult.succeed(resMap);
    }

    /**
     * 学生帐号关联
     *
     * @param info StudentWrapper.UnionStudent
     * @return StudentWrapper.UnionStudentResp
     */
    @Transactional
    @Override
    public StudentWrapper.UnionStudentResp unionStudent(StudentWrapper.UnionStudent info) {

        // 帐号关联响应数据
        StudentWrapper.UnionStudentResp studentResp = StudentWrapper.UnionStudentResp
                .builder()
                .unionId(info.getUnionId())
                .mobile(info.getMobile())
                .updateFlag(false)
                .build();

        // 手机号查询用户信息
        StudentVo studentVo = detailByPhone(info.getMobile());


        com.yonge.cooleshow.biz.dal.entity.SysUser sysUser =
                sysUserMapper.selectOne(Wrappers.<com.yonge.cooleshow.biz.dal.entity.SysUser>lambdaQuery()
                        .eq(com.yonge.cooleshow.biz.dal.entity.SysUser::getPhone, info.getMobile()));

        studentResp.setSysUser(sysUser);
        if (Objects.isNull(studentVo)) {

            if (Objects.isNull(sysUser)) {

                // 自动注册创建用户
                com.yonge.cooleshow.biz.dal.entity.SysUser user = new com.yonge.cooleshow.biz.dal.entity.SysUser();
                user.setUsername(info.getUsername());
                user.setAvatar("https://daya.ks3-cn-beijing.ksyun.com/202203/T1WgJaE.png");
                user.setPhone(info.getMobile());
                user.setLockFlag(0);
                user.setGender(info.getGender());
                user.setUserType(ClientEnum.STUDENT.getCode());
                sysUserMapper.insert(user);

                studentResp.username(user.getUsername()).userId(user.getId()).setSysUser(user);

                // 更新标识
                studentResp.setUpdateFlag(true);
            } else {

                com.yonge.cooleshow.biz.dal.entity.SysUser updateEntity =
                        new com.yonge.cooleshow.biz.dal.entity.SysUser();
                updateEntity.setId(sysUser.getId());
                updateEntity.setUpdateTime(LocalDateTime.now());
                //updateEntity.setDelFlag(Optional.ofNullable(info.getLockFlag()).orElse(0)); // 激活帐号
                if (StringUtils.isBlank(sysUser.getUsername())) {
                    updateEntity.setUsername(info.getUsername());
                }
                // 激活帐号
                sysUserMapper.updateById(updateEntity);

                studentResp.username(sysUser.getUsername()).userId(sysUser.getId());

                Student student = lambdaQuery().eq(Student::getUserId, sysUser.getId()).one();

                if (Objects.nonNull(student)) {
                    // 重置学生隐藏状态
                    info.setLockFlag(0);
                }
                // 更新学生关联帐号
                updateStudentUnionStatus(info, studentResp, student);

            }

            String subjectId = null;
            // 学生声部
            if (!StringUtil.isEmpty(info.getSubjectName())) {
                Subject subject = subjectDao.getSubjectMatchByName(info.getSubjectName());
                if (Objects.nonNull(subject)) {
                    subjectId = String.valueOf(subject.getId());
                }
            }

            // 添加学生帐号
            Student student = new Student();
            student.setUserId(studentResp.getUserId());
            student.setSubjectId(subjectId);
            student.setUnionId(info.getUnionId());
            student.setLockFlag(UserLockFlag.NORMAL);
            student.setHideFlag(Optional.ofNullable(info.getLockFlag()).orElse(0));
            save(student);

        } else {

            studentResp.username(studentVo.getUsername()).userId(studentVo.getUserId());

            Student student = lambdaQuery().eq(Student::getUserId, studentVo.getUserId()).one();

            // 重置学生隐藏状态
            info.setLockFlag(0);
            // 更新学生关联帐号
            updateStudentUnionStatus(info, studentResp, student);
        }

        return studentResp;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean save(StudentWrapper.Student studentInfo) {
        if (studentInfo.getId() == null) {
            return createStudent(studentInfo);
        } else {
            return updateStudent(studentInfo);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void importStudentExcel(List<ExcelDataReaderProperty<StudentWrapper.StudentImport>> dataList,
                                   Long tenantId, Long userId) {
        if (dataList.isEmpty()) {
            throw new BizException("导入数据不能为空");
        }
        dataList.sort(Comparator.comparingInt(ExcelDataReaderProperty::getRowIndex));

        Map<String, Long> subjectIdNamemap = subjectDao.findAll(new HashMap<>())
                .stream().collect(Collectors.toMap(next -> next.getName(), next -> next.getId()));
        Set<String> subjectNames = subjectIdNamemap.keySet();
        // 校验数据的完整性
        List<String> errMsg = new ArrayList<>();
        Map<String, Integer> phoneMap = new HashMap<>();
        int rowIndex = 0;
        for (ExcelDataReaderProperty<StudentWrapper.StudentImport> next : dataList) {
            rowIndex++;
            StudentWrapper.StudentImport student = next.getClazz();

            int msgRowNo = rowIndex + 1;
            student.checkValid().forEach(err -> errMsg.add(String.format("第%s行%s", msgRowNo, err)));

            if (phoneMap.containsKey(student.getPhone().trim())) {
                errMsg.add(String.format("第%s行手机号重复", msgRowNo));
            } else {
                phoneMap.put(student.getPhone().trim(), msgRowNo);
            }

            if (!subjectNames.contains(student.getSubjectName())) {
                errMsg.add(String.format("第%s行声部不支持", msgRowNo));
            }

            if (errMsg.size() > 100) {
                break;
            }
        }

        if (!errMsg.isEmpty()) {
            throw new BizException(String.join(",", errMsg));
        }

        int row = 50;
        int page = 1;
        List<List<String>> phonePartition = Lists.partition(new ArrayList<>(phoneMap.keySet()), row);

        StudentSearch studentSearch = new StudentSearch();
        studentSearch.setRows(row);
        studentSearch.setPage(page);
        for (List<String> phones : phonePartition) {
            studentSearch.setPhoneList(phones);
            List<StudentVo> studentVos = baseMapper.selectPage(PageUtil.getPage(studentSearch), studentSearch);
            page++;

            if (!studentVos.isEmpty()) {
                List<String> existPhoneList = studentVos.stream().map(StudentVo::getPhone).collect(Collectors.toList());
                existPhoneList.forEach(phone -> {
                    if (phoneMap.containsKey(phone)) {
                        errMsg.add(String.format("第%s行手机号已经注册学生", phoneMap.get(phone)));
                    }
                });
            }
        }

        if (!errMsg.isEmpty()) {
            throw new BizException(String.join(",", errMsg));
        }

        for (ExcelDataReaderProperty<StudentWrapper.StudentImport> next : dataList) {
            StudentWrapper.StudentImport studentImport = next.getClazz();
            StudentWrapper.Student student = new StudentWrapper.Student();
            student.setTenantId(tenantId);
            student.setName(studentImport.getUserName());
            student.setGender("男".equals(studentImport.getGender()) ? 1 : 0);
            student.setSubjectId(subjectIdNamemap.get(studentImport.getSubjectName()).toString());
            student.setPhone(studentImport.getPhone());

            LocalDate birthday = LocalDate.parse(studentImport.getBirthday(), DateTimeFormatter.ISO_LOCAL_DATE);
            student.setBirthdate(birthday);
            save(student);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateTenant(Student student, Long toTenantId) {
        Long tenantId = student.getTenantId();
        if (tenantId.equals(toTenantId)) {
            return;
        }
        if (!tenantId.equals(-1L)) {
            List<ImGroupMember> groupMembers = imGroupMemberService.lambdaQuery()
                    .eq(ImGroupMember::getUserId, student.getUserId())
                    .eq(ImGroupMember::getRoleType, ClientEnum.STUDENT.getCode())
                    .eq(ImGroupMember::getIsAdmin, false).list();
            List<String> groupIdList = groupMembers.stream().map(ImGroupMember::getGroupId).distinct()
                    .collect(Collectors.toList());
            for (String groupId : groupIdList) {
                try {
                    imGroupService.quit(groupId, student.getUserId(), ClientEnum.STUDENT, true);
                } catch (Exception e) {
                    log.error("退出群聊失败", e);
                }
            }
            // 删除机构好友关系
            imUserFriendService.delFriendByTenantId(tenantId, student.getUserId(),ClientEnum.STUDENT.getCode());
        }
        if (toTenantId != null && toTenantId != -1L) {
            // 加好友
            QueryWrapper<Teacher> query = new QueryWrapper<>();
            query.lambda().eq(Teacher::getTenantId, toTenantId);
            List<Teacher> teachers = teacherDao.selectList(query);
            for (Teacher teacher : teachers) {
                HashSet<Long> studentIds = new HashSet<>();
                studentIds.add(student.getUserId());
                imUserFriendService.saveUserFriend(teacher.getUserId(), studentIds);
            }
        }
        this.lambdaUpdate().set(Student::getTenantId, toTenantId)
                .eq(Student::getUserId, student.getUserId())
                .update();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addStudent(StudentWrapper.Student student) {
        // 更新头像
        if (StringUtils.isEmpty(student.getAvatar())) {
            String avatar = sysConfigService.findConfigValue(SysConfigConstant.STUDENT_AVATAR);
            student.setAvatar(avatar);
        }
        SysUser sysUser = employeeDao.querySysUserByPhone(student.getPhone());
        if (sysUser == null) {
            sysUser = new SysUser();
            sysUser.setUserType(ClientEnum.STUDENT.getCode());
            sysUser.setGender(student.getGender());
            //设置默认头像
            sysUser.setAvatar(student.getAvatar());

            sysUser.setUsername(student.getName());
            sysUser.setPhone(student.getPhone());
            //初始密码，手机号后6位
            String password = "klx" + sysUser.getPhone().substring(sysUser.getPhone().length() - 4);
            sysUser.setPassword(new BCryptPasswordEncoder().encode(password));
            //插入
            employeeDao.insertSysUser(sysUser);
        } else {
            if(StringUtils.isNotEmpty(sysUser.getUserType()) && sysUser.getUserType().contains(ClientEnum.STUDENT.getCode())){
                throw new BizException("手机号已经注册学生账号");
            }
            if (StringUtil.isEmpty(sysUser.getUserType())) {
                sysUser.setUserType(ClientEnum.STUDENT.getCode());
            } else if (!sysUser.getUserType().contains(ClientEnum.STUDENT.getCode())) {
                sysUser.setUserType(sysUser.getUserType() + "," + ClientEnum.STUDENT.getCode());
            }
            sysUser.setGender(student.getGender());
            employeeDao.updateSysUser(sysUser);
        }

        student.setId(sysUser.getId());
        this.save(student);
    }

    private Boolean updateStudent(StudentWrapper.Student studentInfo) {
        StudentVo student = detail(studentInfo.getId());
        if (student == null) {
            throw new BizException("学生信息不存在");
        }
        // 手机号码修改
        if (!student.getPhone().equals(studentInfo.getPhone())) {
            com.yonge.cooleshow.biz.dal.entity.SysUser sysUser = getOrCreateAccount(studentInfo);
            this.lambdaUpdate().set(Student::getSubjectId, studentInfo.getSubjectId())
                    .set(Student::getTenantId, studentInfo.getTenantId())
                    .set(Student::getUserId, sysUser.getId())
                    .set(Student::getUpdateTime, new Date())
                    .set(StringUtils.isNotEmpty(studentInfo.getAvatar()), Student::getAvatar, studentInfo.getAvatar())
                    .eq(Student::getUserId, studentInfo.getId())
                    .update();
        } else {
            Student newStudent = new Student();
            newStudent.setUserId(studentInfo.getId());
            newStudent.setSubjectId(studentInfo.getSubjectId());
            newStudent.setUpdateTime(new Date());
            newStudent.setTenantId(studentInfo.getTenantId());
            newStudent.setAvatar(studentInfo.getAvatar());
            this.getBaseMapper().updateById(newStudent);
        }
        com.yonge.cooleshow.biz.dal.entity.SysUser sysUser = sysUserMapper.selectById(studentInfo.getId());
        sysUser.setGender(studentInfo.getGender());
        sysUser.setAvatar(studentInfo.getAvatar());
        sysUser.setUsername(studentInfo.getName());
        sysUser.setBirthdate(studentInfo.getBirthdate());
        sysUserMapper.updateById(sysUser);

        try {
            if (StringUtils.isNotEmpty(sysUser.getAvatar())) {
                // 注册IM
                imGroupCoreService.register(String.valueOf(student.getUserId()), ClientEnum.STUDENT.getCode(),
                        sysUser.getUsername(), sysUser.getAvatar());
            }
        } catch (Exception e) {
            log.warn("学生注册IM失败：" + e.getMessage());
        }

        return true;
    }

    private Boolean createStudent(StudentWrapper.Student studentInfo) {
        String avatar = studentInfo.getAvatar();
        if (StringUtils.isEmpty(avatar)) {
            avatar = sysConfigService.findConfigValue(SysConfigConstant.STUDENT_AVATAR);
            studentInfo.setAvatar(avatar);
        }
        com.yonge.cooleshow.biz.dal.entity.SysUser sysUser = getOrCreateAccount(studentInfo);
        Student student = new Student();
        student.setUserId(sysUser.getId());
        student.setTenantId(studentInfo.getTenantId());
        student.setSubjectId(studentInfo.getSubjectId());
        student.setAvatar(studentInfo.getAvatar());
        student.setCreateTime(new Date());
        student.setLockFlag(UserLockFlag.NORMAL);

        save(student);
        try {
            // 注册IM
            imGroupCoreService.register(String.valueOf(student.getUserId()),ClientEnum.STUDENT.getCode(), sysUser.getUsername(), avatar);
        } catch (Exception e) {
            log.warn("学生注册IM失败："+e.getMessage());
        }

        // 加好友
        if (student.getTenantId() != null && student.getTenantId() != -1L) {
            // 自动与机构老师成为好友
            QueryWrapper<Teacher> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(Teacher::getTenantId, student.getTenantId());
            List<Teacher> teacherList = teacherDao.selectList(queryWrapper);
            teacherList.forEach(next -> imUserFriendService.saveUserFriend(next.getUserId(),
                    new HashSet<>(ImmutableList.of(sysUser.getId()))));
        }

        //  与客服建立好友
        String customerService = customerServiceConfig.getCustomerService();
        if(StringUtils.isNotBlank(customerService)){
            List<String> phones = Arrays.stream(customerService.split(",")).collect(Collectors.toList());
            // 通过手机号获取用户ID
            QueryWrapper<com.yonge.cooleshow.biz.dal.entity.SysUser> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().in(com.yonge.cooleshow.biz.dal.entity.SysUser::getPhone,phones);
            List<com.yonge.cooleshow.biz.dal.entity.SysUser> sysUsers = sysUserMapper.selectList(queryWrapper);
            Set<Long> userIds = sysUsers.stream().map(com.yonge.cooleshow.biz.dal.entity.SysUser::getId).collect(Collectors.toSet());
            for (Long teacherId : userIds) {
                HashSet<Long> studentIds = new HashSet<>();
                studentIds.add(student.getUserId());
                imUserFriendService.saveUserFriend(teacherId, studentIds);
            }
        }

        return true;
    }

    private com.yonge.cooleshow.biz.dal.entity.SysUser getOrCreateAccount(StudentWrapper.Student studentInfo) {
        QueryWrapper<com.yonge.cooleshow.biz.dal.entity.SysUser> sysUserQueryWrapper = new QueryWrapper<>();
        sysUserQueryWrapper.lambda().eq(com.yonge.cooleshow.biz.dal.entity.SysUser::getPhone, studentInfo.getPhone());
        List<com.yonge.cooleshow.biz.dal.entity.SysUser> sysUsers = sysUserMapper.selectList(sysUserQueryWrapper);
        com.yonge.cooleshow.biz.dal.entity.SysUser sysUser;
        if (!sysUsers.isEmpty()) {
            sysUser = sysUsers.get(0);
            if (sysUser.getUserType().contains(ClientEnum.STUDENT.getCode())) {
                throw new BizException("手机号已经注册学生账号");
            }
            sysUser.setAvatar(studentInfo.getAvatar());
            sysUser.setGender(studentInfo.getGender());
            sysUser.setUsername(studentInfo.getName());
            sysUser.setBirthdate(studentInfo.getBirthdate());
            String userType = sysUser.getUserType();
            String studentUserType = "STUDENT";
            if (StringUtils.isEmpty(userType)) {
                sysUser.setUserType(studentUserType);
            } else if (!userType.contains(studentUserType)) {
                sysUser.setUserType(userType + "," + studentUserType);
            }
            sysUserMapper.updateById(sysUser);
        } else {
            sysUser = JSON.parseObject(JSON.toJSONString(studentInfo),
                    com.yonge.cooleshow.biz.dal.entity.SysUser.class);
            sysUser.setPhone(studentInfo.getPhone());
            sysUser.setAvatar(studentInfo.getAvatar());
            if (StringUtils.isEmpty(studentInfo.getAvatar())) {
                //设置默认头像
                String avatar = sysConfigService.findConfigValue(SysConfigConstant.STUDENT_AVATAR);
                sysUser.setAvatar(avatar);
            }
            sysUser.setGender(studentInfo.getGender());
            sysUser.setUserType("STUDENT");
            sysUser.setBirthdate(studentInfo.getBirthdate());
            sysUser.setUsername(studentInfo.getName());

            String newPassword = MessageFormat.format("klx{0}", studentInfo.getPhone().substring(7));
            String password = new BCryptPasswordEncoder().encode(newPassword);
            sysUser.setPassword(password);
            sysUserMapper.insert(sysUser);
        }
        return sysUser;
    }

    /**
     * 更新学生关联帐号
     *
     * @param info        StudentWrapper.UnionStudent
     * @param studentResp StudentWrapper.UnionStudentResp
     * @param student     Student
     */
    private void updateStudentUnionStatus(StudentWrapper.UnionStudent info,
                                          StudentWrapper.UnionStudentResp studentResp, Student student) {

        if (Objects.nonNull(student)) {

            if (Optional.ofNullable(student.getUnionId()).orElse(0L) <= 0) {

                // 更新学生关联Id
                lambdaUpdate()
                        .eq(Student::getUserId, student.getUserId())
                        .set(Student::getUnionId, info.getUnionId())
                        .set(Student::getHideFlag, Optional.ofNullable(info.getLockFlag()).orElse(0))
                        .update();
            } else {
                studentResp.setUnionId(student.getUnionId());
            }

            // 更新标识
            studentResp.setUpdateFlag(true);
        }
    }

}