package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dao.StudentDao;
import com.yonge.cooleshow.biz.dal.dto.search.StudentMusicSheetSearch;
import com.yonge.cooleshow.biz.dal.entity.*;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.SubjectTypeEnum;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumMapper;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumPurchaseMapper;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumRefMapper;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.MusicSheetVo;
import com.yonge.cooleshow.biz.dal.vo.StudentVo;
import com.yonge.cooleshow.biz.dal.wrapper.TenantAlbumWrapper;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.base.util.StringUtil;
import com.yonge.toolset.mybatis.support.PageUtil;
import com.yonge.toolset.utils.obj.ObjectUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.beans.BeanUtils;
import lombok.extern.slf4j.Slf4j;
import com.yonge.cooleshow.biz.dal.wrapper.UserTenantAlbumRecordWrapper;
import com.yonge.cooleshow.biz.dal.mapper.UserTenantAlbumRecordMapper;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 购买训练工具记录
 * 2023-07-24 13:32:24
 */
@Slf4j
@Service
public class UserTenantAlbumRecordServiceImpl extends ServiceImpl<UserTenantAlbumRecordMapper, UserTenantAlbumRecord> implements UserTenantAlbumRecordService {

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private TenantAlbumRefService tenantAlbumRefService;

    @Autowired
    private TenantInfoService tenantInfoService;

    @Autowired
    private TenantAlbumMusicService tenantAlbumMusicService;

    @Autowired
    private MusicSheetService musicSheetService;

    @Autowired
    private  StudentService studentService;

    @Autowired
    private TenantAlbumService tenantAlbumService;

    @Autowired
    private UserTenantAlbumRecordMapper userTenantAlbumRecordMapper;

    @Autowired
    private StudentDao studentDao;

    @Autowired
    private TenantAlbumPurchaseMapper tenantAlbumPurchaseMapper;

    @Autowired
    private UserTenantAlbumRecordService userTenantAlbumRecordService;


    @Autowired
    private TenantAlbumRefMapper tenantAlbumRefMapper;

    @Autowired
    private TenantAlbumMapper tenantAlbumMapper;

	/**
     * 查询详情
     * @param id 详情ID
     * @return UserTenantAlbumRecord
     */
	@Override
    public UserTenantAlbumRecord detail(Long id) {
        
        return baseMapper.selectById(id);
    }

    public StudentVo detailStudent(Long userId) {
        return studentDao.detail(userId);
    }

    /**
     * 分页查询
     * @param page IPage<UserTenantAlbumRecord>
     * @param query UserTenantAlbumRecordWrapper.UserTenantAlbumRecordQuery
     * @return IPage<UserTenantAlbumRecord>
     */
    @Override
    public IPage<TenantAlbumWrapper.TenantAlbum> selectPage(IPage<TenantAlbumWrapper.TenantAlbum> page, UserTenantAlbumRecordWrapper.UserTenantAlbumRecordQuery query) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Long id = sysUser.getId();
        query.setUserId(sysUser.getId());

        List<TenantAlbumWrapper.TenantAlbum> list = new ArrayList<>();

            //查询生效的机构专辑id
        if (id != null){
            List<Long> tenantAlbumIds = baseMapper.selectTenantIds(id);
            if (CollectionUtils.isEmpty(tenantAlbumIds)){
                return null;
            }
            List<TenantAlbum> tenantAlbums = baseMapper.selectTenantAlbumInfo(tenantAlbumIds);

            tenantAlbums.stream().forEach(i->{
                TenantAlbumWrapper.TenantAlbum vo = JSON.parseObject(JSON.toJSONString(i),
                        TenantAlbumWrapper.TenantAlbum.class);

                //查关联表
                TenantAlbumRef one = tenantAlbumRefService.lambdaQuery().eq(TenantAlbumRef::getTenantAlbumId, i.getId())
                        .last("limit 1").one();
                vo.setTenantId(one.getTenantId().toString());


                TenantInfo tenantInfo = tenantInfoService.getById(one.getTenantId());
                if (tenantInfo != null) {
                    vo.setTenantName(tenantInfo.getName());
                    vo.setTenantImg(tenantInfo.getLogo());
                }
                //查询机构专辑曲目表
                List<TenantAlbumMusic> tenantAlbumMusics = tenantAlbumMusicService.lambdaQuery()
                        .eq(TenantAlbumMusic::getTenantAlbumId, i.getId())
                        .eq(TenantAlbumMusic::getDelFlag, false)
                        .list();

                Map<SubjectTypeEnum, List<TenantAlbumMusic>> groupByType =
                        tenantAlbumMusics.stream().collect(Collectors.groupingBy(TenantAlbumMusic::getSubjectType));

                List<Long> musicSheetIdlist = tenantAlbumMusics.stream().map(next -> next.getMusicSheetId()).distinct().collect(Collectors.toList());

                StudentMusicSheetSearch search = new StudentMusicSheetSearch();
                search.setMusicSheetIdlist(musicSheetIdlist);
                search.setPage(1);
                search.setRows(9999);

                IPage<MusicSheetVo> records = musicSheetService.selectStudentPage(PageUtil.getPage(search), search, ClientEnum.TENANT_STUDENT);

                Map<Long, MusicSheetVo> idMsMap = records.getRecords().stream()
                        .collect(Collectors.toMap(MusicSheet::getId, Function.identity()));

                List<TenantAlbumWrapper.MusicSheetData> musicSheetData = vo.getMusicSheetData();

                groupByType.forEach((key, value) -> {
                    value.sort(Comparator.comparing(TenantAlbumMusic::getSortNumber));
                    TenantAlbumWrapper.MusicSheetData sheetData = new TenantAlbumWrapper.MusicSheetData();
                    sheetData.setSubjectType(key);
                    List<TenantAlbumWrapper.TenantAlbumSheet> tenantAlbumSheets = value.stream().map(next -> {

                        TenantAlbumWrapper.TenantAlbumSheet tenantAlbumSheet = new TenantAlbumWrapper.TenantAlbumSheet();
                        BeanUtils.copyProperties(next, tenantAlbumSheet);
                        Long musicSheetId = tenantAlbumSheet.getMusicSheetId();
                        MusicSheetVo musicSheet = idMsMap.getOrDefault(musicSheetId, new MusicSheetVo());
                        tenantAlbumSheet.setMusicSheetName(musicSheet.getMusicSheetName());
                        tenantAlbumSheet.setMusicTag(musicSheet.getMusicTag());
                        tenantAlbumSheet.setComposer(musicSheet.getComposer());
                        return tenantAlbumSheet;
                    }).collect(Collectors.toList());


                    sheetData.setTenantAlbumSheetList(tenantAlbumSheets);
                    musicSheetData.add(sheetData);
                    vo.setMusicSheetData(musicSheetData);
                });
                list.add(vo);
            });
        }
        return page.setRecords(list);
    }
	
    /**
     * 添加
     * @param userTenantAlbumRecord UserTenantAlbumRecordWrapper.UserTenantAlbumRecord
     * @return Boolean
     */
    @Override
    public Boolean add(UserTenantAlbumRecordWrapper.UserTenantAlbumRecord userTenantAlbumRecord) {    	
        
        return this.save(JSON.parseObject(userTenantAlbumRecord.jsonString(), UserTenantAlbumRecord.class));
    }

    /**
     * 更新
     * @param userTenantAlbumRecord UserTenantAlbumRecordWrapper.UserTenantAlbumRecord
     * @return Boolean
     */
    @Override
    public Boolean update(UserTenantAlbumRecordWrapper.UserTenantAlbumRecord userTenantAlbumRecord){

        return this.updateById(JSON.parseObject(userTenantAlbumRecord.jsonString(), UserTenantAlbumRecord.class));       
    }

    /**
     * 获取最新的购买记录
     *
     * @param tenantId   机构ID
     * @param userId     用户ID
     * @param clientType 客户端类型
     */
    @Override
    public UserTenantAlbumRecord getNewestByTenantIdAndUserId(Long tenantId, Long userId, ClientEnum clientType) {
        if (tenantId == null || userId == null || clientType == null) {
            return null;
        }
        return this.lambdaQuery()
                .eq(UserTenantAlbumRecord::getTenantId, tenantId)
                .eq(UserTenantAlbumRecord::getUserId, userId)
                .eq(UserTenantAlbumRecord::getClientType, clientType)
                .orderByDesc(UserTenantAlbumRecord::getEndTime)
                .last("limit 1")
                .one();
    }

    @Override
    public List<Long> getUseAlbumIdsByUserId(Long userId,ClientEnum clientType) {
        if (userId == null) {
            return new ArrayList<>();
        }
        List<UserTenantAlbumRecord> list = this.lambdaQuery()
                .eq(UserTenantAlbumRecord::getUserId, userId)
                .eq(UserTenantAlbumRecord::getClientType, clientType)
                .ge(UserTenantAlbumRecord::getEndTime, new Date())
                .list();
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(UserTenantAlbumRecord::getTenantAlbumId).distinct().collect(Collectors.toList());

    }

    @Override
    public TenantAlbumWrapper.TenantAlbum detailAlbum(String albumId) {

        Integer size = 0;
        TenantAlbumWrapper.TenantAlbum album = new TenantAlbumWrapper.TenantAlbum();

        //如果没传专辑id  则查询对应机构的专辑详情
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            throw new BizException("用户不存在");
        }
        Long tenantAlbumId;
        if (StringUtils.isEmpty(albumId)){
            Long id = sysUser.getId();
            List<Student> list = studentService.lambdaQuery().eq(Student::getUserId, id).list();
            if (CollectionUtils.isEmpty(list)) {
                throw new BizException("学生账号未找到");
            }
            Student student = list.get(0);
            //获取机构Id
            Long tenantId = student.getTenantId();
            //查询对应专辑id
            List<TenantAlbumMusic> tenantAlbumMusicList = tenantAlbumMusicService.lambdaQuery().eq(TenantAlbumMusic::getTenantId, tenantId)
                    .eq(TenantAlbumMusic::getDelFlag,false).list();
            if (CollectionUtils.isEmpty(tenantAlbumMusicList)) {
                return null;
            }
            TenantAlbumMusic tenantAlbumMusic = tenantAlbumMusicList.get(0);
             tenantAlbumId = tenantAlbumMusic.getTenantAlbumId();
        } else {
            //如果传专辑id   则查询这个专辑的详情
            tenantAlbumId = Long.parseLong(albumId);
        }
        //获取对应机构专辑状态
        TenantAlbum one = tenantAlbumService.lambdaQuery().eq(TenantAlbum::getId, tenantAlbumId).last("limit 1").one();
        if (!ObjectUtil.isEmpty(one)){
            album.setStatus(one.getStatus());
        }

        StudentVo detail = detailStudent(sysUser.getId());
        // 判断是否是机构学生 机构学生 检测机构专辑购买记录
        TenantInfo tenantInfo = tenantInfoService.detail(detail.getTenantId());
        if (tenantInfo != null) {
            album.setTenantAlbumStatus(0);
            album.setTenantName(tenantInfo.getName());
            QueryWrapper<TenantAlbumRef> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(TenantAlbumRef::getTenantId, detail.getTenantId());
            List<TenantAlbumRef> albumRefs = tenantAlbumRefMapper.selectList(queryWrapper);
            if (!albumRefs.isEmpty()) {
                List<Long> albIds = albumRefs.stream().map(TenantAlbumRef::getTenantAlbumId).distinct().collect(Collectors.toList());
                QueryWrapper<TenantAlbum> query = new QueryWrapper<>();
                query.lambda().in(TenantAlbum::getId, albIds)
                        .eq(TenantAlbum::getStatus,true);
                Integer count = tenantAlbumMapper.selectCount(query);
                if (count > 0) {
                    album.setTenantAlbumStatus(1);
                }
            }
            UserTenantAlbumRecord record =
                    userTenantAlbumRecordService.getNewestByTenantIdAndUserId(tenantInfo.getId(), detail.getUserId(),
                            ClientEnum.STUDENT);
            if (record == null || record.getEndTime().getTime() < System.currentTimeMillis()) {
                album.setTenantAlbumFlag(YesOrNoEnum.NO);
            } else {
                album.setTenantAlbumStatus(2);
                album.setTenantAlbumFlag(YesOrNoEnum.YES);
                album.setTenantAlbumStartTime(record.getStartTime());
                album.setTenantAlbumEndTime(record.getEndTime());
            }
        }




        //查询是否已经购买专辑
        Long buyTenantAlbumId = userTenantAlbumRecordMapper.ifBuy(tenantAlbumId,sysUser.getId());

        if (buyTenantAlbumId != null){
            album.setIfBuy(true);
        } else {
            album.setIfBuy(false);
        }


        //查询对应专辑的详情
        List<TenantAlbum> list = tenantAlbumService.lambdaQuery().eq(TenantAlbum::getId, tenantAlbumId).list();
        if (CollectionUtils.isEmpty(list)){
            throw new BizException("机构专辑不存在");
        }
        TenantAlbum tenantAlbum = list.get(0);
        //机构专辑名称
        String name = tenantAlbum.getName();
        //机构专辑封面
        String coverImg = tenantAlbum.getCoverImg();
        //机构专辑曲目数
        List<TenantAlbumMusic> tenantAlbumMusiclist = tenantAlbumMusicService.lambdaQuery().eq(TenantAlbumMusic::getTenantAlbumId, tenantAlbumId).eq(TenantAlbumMusic::getDelFlag,false).list();
        List<Long> MusicSheetIds = tenantAlbumMusiclist.stream().map(TenantAlbumMusic::getMusicSheetId).distinct().collect(Collectors.toList());
       //计算符合条件的个数
        if (CollectionUtils.isNotEmpty(MusicSheetIds)){
             size = musicSheetService.lambdaQuery().in(MusicSheet::getId, MusicSheetIds).eq(MusicSheet::getState, true)
                    .eq(MusicSheet::getDelFlag, false).count();
        }
        //Integer musicNum = tenantAlbum.getMusicNum();

        //获取合奏曲目数量
        List<TenantAlbumMusic> ensembleLits = tenantAlbumMusicService.lambdaQuery().eq(TenantAlbumMusic::getSubjectType, "ENSEMBLE")
                .eq(TenantAlbumMusic::getTenantAlbumId, tenantAlbumId)
                .eq(TenantAlbumMusic::getDelFlag,false).list();
        List<Long> ensembleMusicSheetIds = ensembleLits.stream().map(TenantAlbumMusic::getMusicSheetId).distinct().collect(Collectors.toList());

        album.setEnsembleCounts(ensembleMusicSheetIds.size());

        //获取小曲目的曲目数量
        List<TenantAlbumMusic> musicLists = tenantAlbumMusicService.lambdaQuery().eq(TenantAlbumMusic::getSubjectType, "MUSIC")
                .eq(TenantAlbumMusic::getTenantAlbumId, tenantAlbumId)
                .eq(TenantAlbumMusic::getDelFlag,false).list();
        List<Long> musicSheetIds = musicLists.stream().map(TenantAlbumMusic::getMusicSheetId).distinct().collect(Collectors.toList());

        album.setMusicCounts(musicSheetIds.size());

        //获取声部的曲目数量
        List<TenantAlbumMusic> subjectLists = tenantAlbumMusicService.lambdaQuery().eq(TenantAlbumMusic::getSubjectType, "SUBJECT")
                .eq(TenantAlbumMusic::getTenantAlbumId, tenantAlbumId)
                .eq(TenantAlbumMusic::getDelFlag,false).list();
        List<Long> subjectSheetIds = subjectLists.stream().map(TenantAlbumMusic::getMusicSheetId).distinct().collect(Collectors.toList());

        album.setSubjectCounts(subjectSheetIds.size());


        //机构专辑简介
        String describe = tenantAlbum.getDescribe();
        //声部
        String subjectTypes = tenantAlbum.getSubjectTypes();
        //原价
        BigDecimal costPrice = tenantAlbum.getCostPrice();
        //售价
        BigDecimal salePrice = tenantAlbum.getSalePrice();
        //周期
        Integer purchaseCycle = tenantAlbum.getPurchaseCycle();

        album.setId(String.valueOf(tenantAlbum.getId()));
        album.setName(name);
        album.setCoverImg(coverImg);
        album.setMusicNum(size);
        album.setDescribe(describe);
        album.setSubjectTypes(subjectTypes);
        album.setCostPrice(costPrice);
        album.setPurchaseCycle(purchaseCycle);
        album.setSalePrice(salePrice);
        return album;
    }
}