package com.yonge.cooleshow.tenant.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.collect.ImmutableList;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dto.search.StudentSearch;
import com.yonge.cooleshow.biz.dal.entity.Student;
import com.yonge.cooleshow.biz.dal.entity.Teacher;
import com.yonge.cooleshow.biz.dal.entity.TenantActivationCode;
import com.yonge.cooleshow.biz.dal.entity.TenantInfo;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.service.ImUserFriendService;
import com.yonge.cooleshow.biz.dal.service.StudentService;
import com.yonge.cooleshow.biz.dal.service.TeacherService;
import com.yonge.cooleshow.biz.dal.service.TenantActivationCodeService;
import com.yonge.cooleshow.biz.dal.service.TenantInfoService;
import com.yonge.cooleshow.biz.dal.vo.StudentVo;
import com.yonge.cooleshow.biz.dal.wrapper.StudentWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.BizHttpStatus;
import com.yonge.cooleshow.common.enums.UserLockFlag;
import com.yonge.cooleshow.common.enums.UserStatusEnum;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import com.yonge.toolset.utils.easyexcel.ErrMsg;
import com.yonge.toolset.utils.easyexcel.ExcelDataReader;
import com.yonge.toolset.utils.easyexcel.ExcelException;
import com.yonge.toolset.utils.easyexcel.ExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/student")
@Api(value = "学生表", tags = "学生表")
public class StudentController extends BaseController {

    @Autowired
    private StudentService studentService;
    @Autowired
    private TeacherService teacherService;
    @Resource
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private ImUserFriendService imUserFriendService;
    @Autowired
    private TenantInfoService tenantInfoService;

    @Autowired
    private TenantActivationCodeService tenantActivationCodeService;

    @GetMapping("/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入id")
    public HttpResponseResult<StudentVo> detail(@PathVariable("id") Long id) {
        StudentVo detail = studentService.detail(id);
        return succeed(detail);
    }

    @PostMapping("/page")
    @ApiOperation(value = "查询分页", notes = "传入StudentSearch")
    public HttpResponseResult<PageInfo<StudentVo>> page(@RequestBody StudentSearch query) {
        TenantInfo tenantInfo = getTenantInfo();
        query.setTenantId(tenantInfo.getId());
        query.setDelFlag(YesOrNoEnum.NO);
        query.setLockFlag(UserLockFlag.NORMAL);
        query.setOrderBy("u.username_ asc");

        IPage<StudentVo> pages = studentService.selectPage(PageUtil.getPage(query), query);
        List<StudentVo> rows = pages.getRecords();

        //如果真实姓名字段为空 把真实姓名赋值给昵称
        rows.stream().forEach(r->{
            if (StringUtils.isEmpty(r.getRealName())){
                if (StringUtils.isNotEmpty(r.getUsername()))
                r.setRealName(r.getUsername());
            }
        });


        Map<String, List<TenantActivationCode>> groupByPhone = new HashMap<>();
        if (!rows.isEmpty() && query.getTenantAlbumPurchaseId() != null) {
            List<String> studentPhones =
                    rows.stream().map(StudentVo::getPhone).filter(StringUtils::isNotEmpty).collect(Collectors.toList());
             groupByPhone = tenantActivationCodeService.lambdaQuery()
                    .eq(TenantActivationCode::getId, query.getTenantAlbumPurchaseId())
                    .in(TenantActivationCode::getActivationPhone, studentPhones)
                    .list().stream().collect(Collectors.groupingBy(TenantActivationCode::getActivationPhone));
        }

        for (StudentVo vo : rows) {
            vo.setSendActiveCodeNum(groupByPhone.getOrDefault(vo.getPhone(), new ArrayList<>()).size());
            if (StringUtils.isNotEmpty(vo.getPhone())) {
                vo.setPhone(vo.getPhone().replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2"));
            }
            if (vo.getDelFlag() == YesOrNoEnum.YES) {
                vo.setUserStatus(UserStatusEnum.CLOSED);
            } else {
                if (vo.getLockFlag() == UserLockFlag.LOCKED) {
                    vo.setUserStatus(UserStatusEnum.LOCKED);
                } else {
                    vo.setUserStatus(UserStatusEnum.NORMAL);
                }
            }
        }
        return succeed(PageUtil.pageInfo(pages));
    }

    @PostMapping("/save")
    @ApiOperation(value = "新增/修改", notes = "传入Student")
    public HttpResponseResult<Boolean> save(
            @Validated @RequestBody com.yonge.cooleshow.tenant.vo.StudentVo.Student student) {
        StudentWrapper.Student studentInfo = JSON.parseObject(JSON.toJSONString(student), StudentWrapper.Student.class);
        TenantInfo tenantInfo = getTenantInfo();
        studentInfo.setTenantId(tenantInfo.getId());

        Long studentId = student.getId();

        if (studentId == null) {
            SysUser sysUser = sysUserFeignService.queryUserByMobile(student.getPhone());
            if (sysUser != null && sysUser.getUserType().contains(ClientEnum.STUDENT.getCode())) {
                studentId = sysUser.getId();
            }
        }

        if (studentId != null) {
            Student one = studentService.lambdaQuery()
                    .eq(Student::getUserId, studentId)
                    .eq(Student::getHideFlag, 0)
                    .last("limit 1").one();
            if (one != null) {
                if (one.getTenantId().equals(-1L)) {
                    throw new BizException("该手机号已经注册为平台学生");
                } else if (!one.getTenantId().equals(tenantInfo.getId())) {
                    throw new BizException("该手机号已经注册为其他平台学生");
                }
            }
        }

        studentService.save(studentInfo);
        return succeed();
    }

    @PostMapping("/importExcel")
    @ApiOperation(value = "导入", notes = "传入file")
    public HttpResponseResult<List<ErrMsg>> importExcel(@RequestParam("file") MultipartFile file) {
        if (null == file) {
            return HttpResponseResult.failed("请上传文件");
        }
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        try {
            ExcelDataReader<StudentWrapper.StudentImport> reader =
                    ExcelUtils.getReader(StudentWrapper.StudentImport.class, file);
            studentService.importStudentExcel(reader.getDataList(), user.getTenantId(), user.getId());
            return HttpResponseResult.succeed();
        } catch (ExcelException e) {
            log.error("导入学生失败：" + JSON.toJSONString(e.getErrMsgList()));
            return HttpResponseResult.failed("文件解析失败");
        }
    }

    private TenantInfo getTenantInfo() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        TenantInfo tenantInfo = tenantInfoService.getById(sysUser.getTenantId());
        if (tenantInfo == null) {
            throw new BizException("非法请求");
        }
        return tenantInfo;
    }
}